package de.mattv.fileserver.data;

import de.mattv.fileserver.security.TokenService;
import de.mattv.fileserver.util.AutoCloseLock;
import de.mattv.fileserver.util.NonNull;
import org.springframework.util.FileSystemUtils;

import java.io.File;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicLong;
import java.util.concurrent.locks.ReentrantReadWriteLock;

public class User {
    public final long id;
    public final @NonNull String name;
    public @NonNull String password;

    public boolean enabled = false;
    public boolean admin = false;

    public String tfaSecret = null;
    public String tempTfaSecret = null;

    public final AtomicLong nextNodeId = new AtomicLong(1);

    public final Node rootNode = new Node(0, "/", this);
    public final ConcurrentHashMap<Long, Node> nodes = new ConcurrentHashMap<>();
    public final ReentrantReadWriteLock nodeLock = new ReentrantReadWriteLock();
    public final File userDir;

    public User(long id, @NonNull String name, @NonNull String password) {
        this.id = id;
        this.name = name;
        this.password = password;
        this.nodes.put(0L, rootNode);
        this.userDir = new File(Data.FILES_DIR, String.valueOf(id));

        if (!userDir.exists()) if (!userDir.mkdir()) throw new RuntimeException("Failed to create user " + id + " directory");
    }

    @Override
    public String toString() {
        return "User{" +
                "id=" + id +
                ", name='" + name + '\'' +
                ", admin=" + admin +
                '}';
    }

    public void delete() {
        enabled = false;
        TokenService.logoutAll(id);
        nodeLock.writeLock().lock();
        try (AutoCloseLock ignored = new AutoCloseLock(Data.USER_LOCK.writeLock())) {
            Data.USERS.remove(id);
        }
        for (Node node : nodes.values()) {
            if (node.isFile) {
                boolean ignored = node.file.delete();
                if (node.hasPreview) {
                    boolean ignored2 = node.previewFile.delete();
                }
            }
            node.parent = null;
            node.children.clear();
        }
        FileSystemUtils.deleteRecursively(userDir);
    }
}
