package de.mattv.fileserver.data.converter;

import com.thoughtworks.xstream.io.HierarchicalStreamReader;
import lombok.extern.slf4j.Slf4j;

import java.util.Stack;

@Slf4j
public class ConverterUtils {
    protected static final Stack<String> CONVERSION_STACK_FRAMES = new Stack<>();
    public static boolean hasError = false;

    protected static void dumpConversionStackFrames() {
        log.error("Conversion stack: {}", String.join(" > ", CONVERSION_STACK_FRAMES));
        hasError = true;
    }

    protected static String getString(HierarchicalStreamReader reader, String name) {
        String attribute = reader.getAttribute(name);
        if (attribute == null) {
            log.error("Missing required attribute: {}", name);
            dumpConversionStackFrames();
            attribute = "";
        }
        return attribute;
    }

    protected static long getLong(HierarchicalStreamReader reader, String name) {
        String attribute = getString(reader, name);
        try {
            return Long.parseLong(attribute);
        } catch (NumberFormatException e) {
            log.error("Can't convert attribute to long: {} '{}'", name, attribute);
            dumpConversionStackFrames();
            return 0;
        }
    }

    protected static void push(String frame) { CONVERSION_STACK_FRAMES.push(frame); }
    protected static void pop() { CONVERSION_STACK_FRAMES.pop(); }
}
