package de.mattv.fileserver.routes.admin;

import de.mattv.fileserver.Response;
import de.mattv.fileserver.data.Data;
import de.mattv.fileserver.data.Token;
import de.mattv.fileserver.security.TokenService;
import de.mattv.fileserver.util.AdminRestController;
import de.mattv.fileserver.util.AutoCloseLock;
import io.swagger.v3.oas.annotations.Parameter;
import de.mattv.fileserver.util.NonNull;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.List;

@AdminRestController
public class User {
    @PostMapping("/users")
    private @NonNull List<Response.UserInfo> listUsers() {
        return Data.USERS.values().stream()
                .map(user -> new Response.UserInfo(
                        user.id,
                        user.name,
                        user.tfaSecret != null,
                        user.enabled,
                        user.admin
                ))
                .toList();
    }

    @PostMapping("/user/logout")
    private void logout(@RequestBody long id) {
        TokenService.logoutAll(id);
    }

    @PostMapping("/user/disable_tfa")
    private void disableTfa(@RequestBody long id) {
        try (AutoCloseLock ignored = new AutoCloseLock(Data.USER_LOCK.readLock())) {
            de.mattv.fileserver.data.User user = Data.USERS.get(id);
            if (user != null) user.tfaSecret = "";
            Data.save();
        }
    }

    private record BodySetProp(@NonNull long id, @NonNull boolean state) {}

    @PostMapping("/user/set_enabled")
    private void setEnabled(@RequestBody BodySetProp body) {
        try (AutoCloseLock ignored = new AutoCloseLock(Data.USER_LOCK.readLock())) {
            de.mattv.fileserver.data.User user = Data.USERS.get(body.id);
            if (user != null) user.enabled = body.state;
            Data.save();
        }
    }

    @PostMapping("/user/set_admin")
    private void setAdmin(@RequestBody BodySetProp body) {
        try (AutoCloseLock ignored = new AutoCloseLock(Data.USER_LOCK.readLock())) {
            de.mattv.fileserver.data.User user = Data.USERS.get(body.id);
            if (user != null) user.admin = body.state;
            Data.save();
        }
    }

    @PostMapping("/user/sudo")
    private void sudo(@Parameter(hidden = true) Token token, @RequestBody long id) {
        try (AutoCloseLock ignored = new AutoCloseLock(Data.USER_LOCK.readLock())) {
            de.mattv.fileserver.data.User user = Data.USERS.get(id);
            if (user != null) token.sudo(user);
        }
    }
    @PostMapping("/user/delete")
    private void delete(@RequestBody long id) {
        try (AutoCloseLock ignored = new AutoCloseLock(Data.USER_LOCK.readLock())) {
            de.mattv.fileserver.data.User user = Data.USERS.get(id);
            if (user != null) user.delete();
        }
    }
}
