package de.mattv.fileserver.routes.auth;


import de.mattv.fileserver.Response;
import de.mattv.fileserver.data.Data;
import de.mattv.fileserver.data.User;
import de.mattv.fileserver.security.TokenService;
import de.mattv.fileserver.services.TFAMailer;
import de.mattv.fileserver.util.AuthUser;
import de.mattv.fileserver.util.UserRestController;
import dev.samstevens.totp.secret.DefaultSecretGenerator;
import dev.samstevens.totp.secret.SecretGenerator;
import de.mattv.fileserver.util.NonNull;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.Optional;

@UserRestController
public class TFA {
    private static final SecretGenerator TOTP_SECRET_GENERATOR = new DefaultSecretGenerator();

    private final TFAMailer mailer;

    public TFA(TFAMailer mailer) {
        this.mailer = mailer;
    }

    @PostMapping("/tfa/setup_mail")
    private @NonNull Optional<String> setupTfaMail(@AuthUser User user) {
        if (user.tfaSecret != null)
            return Optional.of("Tfa is already enabled");

        user.tempTfaSecret = "";
        mailer.sendMail(user);
        return Optional.empty();
    }

    @PostMapping("/tfa/setup_totp")
    private @NonNull Response<String> setupTfaTotp(@AuthUser User user) {
        if (user.tfaSecret != null)
            return Response.e("Tfa is already enabled");

        user.tempTfaSecret = TOTP_SECRET_GENERATOR.generate();
        return Response.o(user.tempTfaSecret);
    }

    @PostMapping("/tfa/complete")
    private @NonNull Optional<String> setupComplete(@AuthUser User user, @RequestBody @NonNull String otp) {
        if (user.tempTfaSecret == null)
            return Optional.of("You never started tfa setup");

        boolean ok = user.tempTfaSecret.isEmpty()
                ? mailer.checkCode(user.id, otp)
                : AuthUtils.TOTP_VERIFIER.isValidCode(user.tempTfaSecret, otp);
        if (!ok)
            return Optional.of("Invalid code");

        user.tfaSecret = user.tempTfaSecret;
        user.tempTfaSecret = null;
        Data.save();
        TokenService.logoutAll(user.id);
        return Optional.empty();
    }

    @PostMapping("/tfa/disable")
    private void disableTfa(@AuthUser User user) {
        user.tfaSecret = null;
        Data.save();
        TokenService.logoutAll(user.id);
    }
}
