package de.mattv.fileserver.routes.fs;

import de.mattv.fileserver.Response;
import de.mattv.fileserver.data.Node;
import de.mattv.fileserver.data.User;
import de.mattv.fileserver.util.AuthUser;
import de.mattv.fileserver.util.AutoCloseLock;
import de.mattv.fileserver.util.UserRestController;
import de.mattv.fileserver.util.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.io.IOException;
import java.util.Optional;

@Slf4j
@UserRestController
public class Create {
    private record Body(@NonNull String name, @NonNull long parent, @NonNull boolean file) {}

    @PostMapping("/fs/create")
    private @NonNull Response<Response.CreateNodeInfo> create(@AuthUser User user, @RequestBody Body body) {
        try (AutoCloseLock ignored = new AutoCloseLock(user.nodeLock.readLock())) {
            Node parent = user.nodes.get(body.parent);
            if (parent == null) return Response.e("Invalid parent node");

            Optional<Node> existing = parent.children.stream().filter(n -> n.name.equals(body.name)).findFirst();
            if (existing.isPresent()) return Response.o(new Response.CreateNodeInfo(
                    existing.get().id,
                    true,
                    existing.get().isFile
            ));

            long id = user.nextNodeId.getAndIncrement();
            Node node = new Node(id, body.name, user);
            if (body.file) {
                boolean ignored2 = node.file.createNewFile();
            }
            node.isFile = body.file;
            parent.children.add(node);
            node.parent = parent;
            user.nodes.put(id, node);
            return Response.o(new Response.CreateNodeInfo(
                    id,
                    false,
                    body.file
            ));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
