package de.mattv.fileserver.security;

import io.swagger.v3.oas.annotations.enums.SecuritySchemeType;
import io.swagger.v3.oas.annotations.security.SecurityScheme;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configurers.AbstractHttpConfigurer;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.security.web.context.NullSecurityContextRepository;

@EnableWebSecurity
@Configuration
@SecurityScheme(
        name = "Token",
        type = SecuritySchemeType.HTTP,
        scheme = "bearer"
)
public class Config {
    @Bean
    public SecurityFilterChain securityFilterChain(HttpSecurity http) throws Exception {
        Manager manager = new Manager();
        Filter filter = new Filter();
        filter.setAuthenticationManager(manager);
        filter.setSecurityContextRepository(new NullSecurityContextRepository());

        return http
                .addFilter(filter)
                .authenticationManager(manager)
                .securityMatcher("/api/user/**", "/api/admin/**")
                .authorizeHttpRequests(auth -> {
                            auth.requestMatchers("/api/admin/**").hasRole("ADMIN");
                            auth.anyRequest().authenticated();
                        }
                )
                .cors(AbstractHttpConfigurer::disable)
                .csrf(AbstractHttpConfigurer::disable)
                .sessionManagement(s -> s.sessionCreationPolicy(SessionCreationPolicy.STATELESS))
                .build();
    }

    @Bean
    public UserDetailsService userDetailsService() {
        return username -> null;
    }
}
